/*
    GDAM - Geoff & Dave's Audio Mixer
    Copyright (C) 1999    Dave Benson, Geoff Matters.

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307  USA

    Contact:
        daveb@ffem.org <Dave Benson>
        geoff@ugcs.caltech.edu <Geoff Matters>
*/
#include "gdamsampleeditor.h"

/* 
 * TODO:
 *
 *  - redraw call when data changes
 *  - commands to make new buffers.
 */

static GtkObjectClass* parent_class;

static void     gdam_sample_editor_play    (GdamProducer*    producer,
					    GdamPlayInfo*    play_info);
static void     gdam_sample_editor_destroy (GtkObject*       object);


static void gdam_sample_editor_init(GdamSampleEditor* sample_editor)
{
}

static void gdam_sample_editor_class_init(GdamSampleEditorClass* klass)
{
	GDAM_PRODUCER_CLASS(klass)->play = gdam_sample_editor_play;
	GTK_OBJECT_CLASS(klass)->destroy = gdam_sample_editor_destroy;
}

GtkType gdam_sample_editor_get_type()
{
	static GtkType sample_editor_type = 0;
	if (!sample_editor_type) {
		static const GtkTypeInfo sample_editor_info = {
			"GdamSampleEditor",
			sizeof(GdamSampleEditor),
			sizeof(GdamSampleEditorClass),
			(GtkClassInitFunc) gdam_sample_editor_class_init,
			(GtkObjectInitFunc) gdam_sample_editor_init,
			/* reserved_1*/ NULL,
			/* reserved_2*/ NULL,
			(GtkClassInitFunc) NULL
		};
		GtkType parent = GDAM_TYPE_PRODUCER;
		parent_class = gtk_type_class(parent);
		sample_editor_type = gtk_type_unique(parent,
		                                 &sample_editor_info);
	}
	return sample_editor_type;
}

static void     gdam_sample_editor_play    (GdamProducer*    producer,
					    GdamPlayInfo*    play_info)
{
	int local_source_id;
	GdamRequestBuilder* request_builder;
	GdamRequest* request;
	GdamModel* component = GDAM_MODEL(producer);
	GdamSampleEditor* editor = GDAM_SAMPLE_EDITOR(producer);
	if (editor->buffer_id == 0)
		return;

	request = gdam_request_create_source(
			"GdamWavetable",
			gdam_model_got_subsource,
			gdam_model_didnt_get_subsource,
			producer,
			3,
			"buffer_id",
			GDAM_ARG_TYPE_INT,
			editor->buffer_id,
			"position",
			GDAM_ARG_TYPE_INT64,
			(gint64) play_info->position,
			"terminates",
			GDAM_ARG_TYPE_INT,
			play_info->terminates);
	request_builder = gdam_request_builder_new();
	local_source_id = gdam_request_builder_append_create(request_builder,
	                                                     request);

	gdam_model_new_child_request(component->parent,
	                                 component,
				         local_source_id,
				         request_builder);
}

static void     gdam_sample_editor_destroy (GtkObject*       object)
{
}
