/*
    GDAM - Geoff & Dave's Audio Mixer
    Copyright (C) 1999    Dave Benson, Geoff Matters.

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307  USA

    Contact:
        daveb@ffem.org <Dave Benson>
        geoff@ugcs.caltech.edu <Geoff Matters>
*/
#include "gdammodulator.h"
#include <gdam/gdamnamespace.h>

static void gdam_modulator_init(GdamMix* mix)
{
}

/* XXX: this function needs a C level interface. */
static void gdam_modulator_arg_set_filtered(GtkObject* object, GdamArg* arg)
{
	GdamSource* source;
	GdamMix* mix = GDAM_MIX(object);
	int source_id = GDAM_ARG_INT(arg);
	if (source_id != 0) {
		source = gdam_namespace_local_source(source_id);
		if (source == NULL) {
			gdam_debug("modulator_set_filtered: "
			           "no source of id %d", source_id);
			return;
		}
	} else {
		source = NULL;
	}
	gdam_mix_connect_named(mix, source, -1, "filtered");
}
static void gdam_modulator_arg_get_filtered(GtkObject* object, GdamArg* arg)
{
	GdamSource* filtered;
	filtered = gdam_mix_get_named(GDAM_MIX(object), "filtered");
	GDAM_ARG_INT(arg) = filtered ? filtered->source_id : 0;
}

static void gdam_modulator_arg_set_modulator(GtkObject* object, GdamArg* arg)
{
	GdamSource* source;
	GdamMix* mix = GDAM_MIX(object);
	int source_id = GDAM_ARG_INT(arg);
	if (source_id != 0) {
		source = gdam_namespace_local_source(source_id);
		if (source == NULL) {
			gdam_debug("modulator_set_modulator: "
			           "no source of id %d", source_id);
			return;
		}
	} else {
		source = NULL;
	}
	gdam_mix_connect_named(mix, source, -1, "modulator");
}
static void gdam_modulator_arg_get_modulator(GtkObject* object, GdamArg* arg)
{
	GdamSource* mod;
	mod = gdam_mix_get_named(GDAM_MIX(object), "modulator");
	GDAM_ARG_INT(arg) = mod ? mod->source_id : 0;
}

static void gdam_modulator_class_init(GdamMixClass* mix_class)
{
	GtkObjectClass* object_class = GTK_OBJECT_CLASS(mix_class);
	mix_class->augment =
		(void (*)(GdamSample*, const GdamSample*, int, gpointer))
				gdam_sample_modulate;
	gdam_arg_register(object_class,
	                  "filtered",
			  GDAM_ARG_TYPE_INT,
			  gdam_modulator_arg_set_filtered,
			  gdam_modulator_arg_get_filtered,
			  GDAM_ARG_VITAL);
	gdam_arg_register(object_class,
	                  "modulator",
			  GDAM_ARG_TYPE_INT,
			  gdam_modulator_arg_set_modulator,
			  gdam_modulator_arg_get_modulator,
			  GDAM_ARG_VITAL);
}

GtkType gdam_modulator_get_type()
{
	static GtkType modulator_type = 0;
	if (!modulator_type) {
		static const GtkTypeInfo modulator_info = {
			"GdamModulator",
			sizeof(GdamMix),
			sizeof(GdamMixClass),
			(GtkClassInitFunc) gdam_modulator_class_init,
			(GtkObjectInitFunc) gdam_modulator_init,
			/* reserved_1*/ NULL,
			/* reserved_2*/ NULL,
			(GtkClassInitFunc) NULL
		};
		modulator_type = gtk_type_unique(GDAM_TYPE_MIX,
		                                 &modulator_info);
	}
	return modulator_type;
}
